% 3D Noughts and Crosses AI Program
% Copyright Michael Hawes 2009

% flattens a list
% 1 - terminating - given list is empty
flatten([],[]).
% 2 - recursive - head is not a list
flatten([A|L],[A|L1]) :- 
     xatom(A), flatten(L,L1).
% 3 - recursive - head is a list
flatten([A|L],R) :- 
     flatten(A,A1), flatten(L,L1), append(A1,L1,R).

% succeeds if a variable is an atom or number
xatom(A) :- atom(A).
xatom(A) :- number(A).


% append two lists
% 1 - terminating - first list is empty
append([], List, List).
% 2 - recursive - first list is not empty
append([H|T], List, [H|New]) :-
	append(T, List, New).


% succeeds if an item is a member of a list
% 1 - terminating - given item matches with the head of the list
member(Item, [Item|_]).
% 2 - recursive - given item does not match with the head of the list
member(Item, [_|Tail]) :-
	member(Item, Tail).

% counts the number of elements in a list
% 1 - terminating - list is empty
count([],0).
% 2 - recursive - list is not empty
count([_|R], N) :- count(R,N1), N is N1 + 1.


% returns all lists in a given list of lists that contain a given item
% 1 - terminating - given list is empty
containing([], _Item, []).
% 2 - recursive - list at head contains the given item
containing([Head|Tail1], Item, [Head|Tail2]) :-
	member(Item, Head),
	containing(Tail1, Item, Tail2).
% 3 - recursive - list at head does not contain the given item
containing([Head|Tail], Item, List2) :-
	\+ member(Item, Head),
	containing(Tail, Item, List2).


% writes all elements in a list, each on a seperate line
% 1 - terminating - list is empty
writeList([]).
% 2 - recursive - list is not empty
writeList([Head|Tail]) :-
	write(Head),
	nl,
	writeList(Tail).


% succeeds if the given triplet is a member of the given list
% 1 - terminating - triplet found in the list
memberTriplet(Triplet, [Triplet|_]).
% 2 - terminating - opposite triplet found in the list (still the same triplet)
memberTriplet([Square1,Square2,Square3], [[Square3,Square2,Square1]|_]).
% 3 - recursive - head is not the triplet we're looking for
memberTriplet(Triplet, [_|Tail]) :-
	memberTriplet(Triplet, Tail).


% returns all duplicate triplets in the given list of triplets
% 1 - terminating - list is empty
getDuplicateTriplets([], []).
% 2 - recursive - head is duplicated in the rest of the list
getDuplicateTriplets([Head|Tail1], [Head|Tail2]) :-
	memberTriplet(Head, Tail1),
	getDuplicateTriplets(Tail1, Tail2).
% 3 - recursive - head is not duplicated in the rest of the list
getDuplicateTriplets([Head|Tail], List) :-
	\+ memberTriplet(Head, Tail),
	getDuplicateTriplets(Tail, List).


% specifies the ASCII value of the new line character
newline(10).


% the possible positions in a triplet
tripletPos(1).
tripletPos(2).
tripletPos(3).


% succeeds if a valid square is given
square([X,Y,Z]) :-
	tripletPos(X),
	tripletPos(Y),
	tripletPos(Z).


% returns the computer's chosen square for the move
compMove(Board, Square) :-
	getTriplets(Triplets),
	pickSquare(Board, Triplets, Square).


% chooses a square from the triplets given
% 1 - there is a triplet that has two computer marks, so make the winning move
pickSquare(Board, Triplets, Square) :-
	findTripletWithMarks(Board, Triplets, 0, 2, Triplet),
	findFreeSquare(Board, Triplet, Square).
	%write('Chose square to win, ').
% 2 - there is a triplet that has two player marks, so block their winning move
pickSquare(Board, Triplets, Square) :-
	findTripletWithMarks(Board, Triplets, 2, 0, Triplet),
	findFreeSquare(Board, Triplet, Square).
	%write('Chose square to block player, ').
% 4 - rate all squares on the board and pick the best choice
pickSquare(Board, _Triplets, Square) :-
	findHighestRatedSquare(Board, Square).
	%write('Chose rated square, ').


% returns the highest rated empty square on the board
% 1 - calling - finds the list of empty squares and makes accumulators
findHighestRatedSquare(Board, Square) :-
	findall(Item, emptySquare(Board, Item), Squares),
	findHighestRatedSquare(Board, Squares, 0, 1, 0, Square).
% 2 - terminating - there are no empty squares left to rate
findHighestRatedSquare(_Board, [], _Rating, _RatePerTrip, Square, Square).
% 3 - recursive - there is a square to rate, and its per trip rating is sufficient to make it the max
findHighestRatedSquare(Board, [Head|Tail], _MaxRating, MaxPerTrip, _MaxSquare, Square) :-
	rateSquare(Board, Head, _Rating, PerTripRating),
	PerTripRating > MaxPerTrip,
	findHighestRatedSquare(Board, Tail, 20, PerTripRating, Head, Square).
% 3 - recursive - there is a square to rate, and its rating is greater than the current max
findHighestRatedSquare(Board, [Head|Tail], MaxRating, MaxPerTrip, _MaxSquare, Square) :-
	rateSquare(Board, Head, Rating, _PerTripRating),
	Rating > MaxRating,
	findHighestRatedSquare(Board, Tail, Rating, MaxPerTrip, Head, Square).
% 4 - recursive - there is a square to rate, and its rating is equal to or worse than the current max
findHighestRatedSquare(Board, [Head|Tail], MaxRating, MaxPerTrip, MaxSquare, Square) :-
	rateSquare(Board, Head, Rating, _PerTripRating),
	Rating =< MaxRating,
	findHighestRatedSquare(Board, Tail, MaxRating, MaxPerTrip, MaxSquare, Square).


% checks if a square is empty (used to find all empty squares)
emptySquare(Board, Square) :-
	square(Square),
	marked(Board, Square, nil).


% returns a rating for an empty square (find triplets that contain it, then find their combined rating)
rateSquare(Board, Square, Rating, PerTripRating) :-
	findTriplets(Square, Triplets),
	rateTriplets(Board, Triplets, Rating),
	count(Triplets, NumTrips),
	PerTripRating is Rating / NumTrips.

testPerTripRating1 :-
	assertMarks(comp),
	Board = [[[0,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,x,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]],
	drawBoard(Board),
	rateSquare(Board, [3,2,3], Rating, PerTripRating),
	write('[3,2,3]:'), nl,
	write('Rating = '), write(Rating), nl,
	write('PerTripRating = '), write(PerTripRating), nl,
	retractMarks(comp).

testPerTripRating2 :-
	assertMarks(comp),
	Board = [[[0,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,x,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]],
	drawBoard(Board),
	rateSquare(Board, [1,1,3], Rating, PerTripRating),
	write('[3,2,3]:'), nl,
	write('Rating = '), write(Rating), nl,
	write('PerTripRating = '), write(PerTripRating), nl,
	retractMarks(comp).


% returns a combined rating for a list of triplets, that each have
% at least one empty square and at most one of each mark
% 1 - calling - creates an accumulator to store current total rating
rateTriplets(Board, Triplets, Rating) :-
	rateTriplets(Board, Triplets, 0, Rating).
% 2 - terminating - no triplets left to rate
rateTriplets(_Board, [], Rating, Rating).
% 3 - recursive - there is at least one triplet left to rate, rate it and add the rating to the accumulator
rateTriplets(Board, [Triplet|Tail], Acc, Rating) :-
	rateTriplet(Board, Triplet, Rate),
	NewAcc is Acc + Rate,
	rateTriplets(Board, Tail, NewAcc, Rating).


% returns a rating for a triplet that has at least one empty square
% and can't have two of the same mark in it
% 1 - triplet has a player mark in it, rating is 0
rateTriplet(Board, Triplet, 0) :-
	playerMark(PlayerMark),
	numMarks(Board, Triplet, PlayerMark, 1).
% 2 - triplet has no marks of any kind in it, rating is 1
rateTriplet(Board, Triplet, 1) :-
	compMark(CompMark),
	numMarks(Board, Triplet, CompMark, 0),
	playerMark(PlayerMark),
	numMarks(Board, Triplet, PlayerMark, 0).
% 3 - triplet has a computer mark and no player marks in it, rating is 2
rateTriplet(Board, Triplet, 2) :-
	compMark(CompMark),
	numMarks(Board, Triplet, CompMark, 1),
	playerMark(PlayerMark),
	numMarks(Board, Triplet, PlayerMark, 0).


% finds the first free square in the list of squares given
% 1 - terminating - free square has been found
findFreeSquare(Board, [Square|_Tail], Square) :-
	marked(Board, Square, nil).
findFreeSquare(Board, [Head|Tail], Square) :-
% 2 - recursive - head is not free, search the tail
	marked(Board, Head, Mark),
	Mark \== nil,
	findFreeSquare(Board, Tail, Square).


% returns a triplet in the given list that has the given number of
% player and computer marks
% 1 - terminating - head triplet has correct number of player marks and computer marks
findTripletWithMarks(Board, [Triplet|_Tail], PlayerMarks, CompMarks, Triplet) :-
	playerMark(PlayerMark),
	numMarks(Board, Triplet, PlayerMark, PlayerMarks),
	compMark(CompMark),
	numMarks(Board, Triplet, CompMark, CompMarks).
% 2 - recursive - head triplet has incorrect number of player marks, correct number of computer marks
findTripletWithMarks(Board, [Head|Tail], PlayerMarks, CompMarks, Triplet) :-
	playerMark(PlayerMark),
	\+ numMarks(Board, Head, PlayerMark, PlayerMarks),
	compMark(CompMark),
	numMarks(Board, Head, CompMark, CompMarks),
	findTripletWithMarks(Board, Tail, PlayerMarks, CompMarks, Triplet).
% 3 - recursive - head triplet has incorrect number of computer marks, correct number of player marks
findTripletWithMarks(Board, [Head|Tail], PlayerMarks, CompMarks, Triplet) :-
	playerMark(PlayerMark),
	numMarks(Board, Head, PlayerMark, PlayerMarks),
	compMark(CompMark),
	\+ numMarks(Board, Head, CompMark, CompMarks),
	findTripletWithMarks(Board, Tail, PlayerMarks, CompMarks, Triplet).
% 4 - recursive - head triplet has incorrect number of computer marks, incorrect number of player marks
findTripletWithMarks(Board, [Head|Tail], PlayerMarks, CompMarks, Triplet) :-
	playerMark(PlayerMark),
	\+ numMarks(Board, Head, PlayerMark, PlayerMarks),
	compMark(CompMark),
	\+ numMarks(Board, Head, CompMark, CompMarks),
	findTripletWithMarks(Board, Tail, PlayerMarks, CompMarks, Triplet).

testFindTripletWithMarks1 :-
	assert(playerMark(x)),
	assert(compMark(0)),
	getTriplets(Triplets),
	findTripletWithMarks([[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
			      [[nil,nil,nil],[nil,x,nil],[nil,nil,nil]],
			      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]], Triplets, 1, 0, Triplet),
	write(Triplet).


% returns the number of the given mark in the given triplet
% 1 - calling - creates an accumulator for the number of the given marks so far
numMarks(Board, Triplet, Mark, NumMarks) :-
	numMarks(Board, Triplet, Mark, 0, NumMarks).
% 1 - terminating - no squares left in the triplet
numMarks(_Board, [], _Mark, NumMarks, NumMarks).
% 2 - recursive - current square has the given mark
numMarks(Board, [Square|Tail], Mark, Acc, NumMarks) :-
	marked(Board, Square, Mark),
	NewAcc is Acc + 1,
	numMarks(Board, Tail, Mark, NewAcc, NumMarks).
% 3 - recursive - current square doesn't have the given mark
numMarks(Board, [Square|Tail], Mark, Acc, NumMarks) :-
	\+ marked(Board, Square, Mark),
	numMarks(Board, Tail, Mark, Acc, NumMarks).


% returns whether or not the given square is marked with the given mark
% 1 - X is 1, check the correct list
marked([XList1, _XList2, _XList3], [1,Y,Z], Mark) :-
	markedXList(XList1, [Y,Z], Mark).
% 2 - X is 2, check the correct list
marked([_XList1, XList2, _XList3], [2,Y,Z], Mark) :-
	markedXList(XList2, [Y,Z], Mark).
% 3 - X is 3, check the correct list
marked([_XList1, _XList2, XList3], [3,Y,Z], Mark) :-
	markedXList(XList3, [Y,Z], Mark).


% 1 - Y is 1, check the correct list
markedXList([YList1, _YList2, _YList3], [1,Z], Mark) :-
	markedSquare(YList1, Z, Mark).
% 2 - Y is 2, check the correct list
markedXList([_YList1, YList2, _YList3], [2,Z], Mark) :-
	markedSquare(YList2, Z, Mark).
% 3 - Y is 3, check the correct list
markedXList([_YList1, _YList2, YList3], [3,Z], Mark) :-
	markedSquare(YList3, Z, Mark).


% 1 - Z is 1, check the square
markedSquare([Mark, _Z2, _Z3], 1, Mark).
% 2 - Z is 2, check the square
markedSquare([_Z1, Mark, _Z3], 2, Mark).
% 3 - Z is 3, check the square
markedSquare([_Z1, _Z2, Mark], 3, Mark).


% gets a list of all the triplets
getTriplets(Triplets) :-
	findall(StraightTriplet, straightTriplet(StraightTriplet), Triplets1),
	findall(DiagTriplet, diagTriplet(DiagTriplet), Triplets2),
	append(Triplets1, Triplets2, Triplets).

testGetTriplets :-
	getTriplets(Triplets),
	getDuplicateTriplets(Triplets, Duplicates),
	writeList(Triplets), nl,
	write('Duplicates: '), nl,
	writeList(Duplicates).


% returns a list of all triplets that contain the given square
findTriplets(Square, Triplets) :-
	findall(StraightTriplet, straightTriplet(Square, StraightTriplet), Triplets1),
	findall(DiagTriplet, diagTriplet(DiagTriplet), DiagTriplets),
	containing(DiagTriplets, Square, Triplets2),
	append(Triplets1, Triplets2, Triplets).

testFindTriplets(Square) :-
	findTriplets(Square, Triplets),
	writeList(Triplets).


% defines the straight triplets that include a given square
straightTriplet([X,Y,Z], [[1,Y,Z],[2,Y,Z],[3,Y,Z]]) :-
	tripletPos(X),
	tripletPos(Y),
	tripletPos(Z).
straightTriplet([X,Y,Z], [[X,1,Z],[X,2,Z],[X,3,Z]]) :-
	tripletPos(X),
	tripletPos(Y),
	tripletPos(Z).
straightTriplet([X,Y,Z], [[X,Y,1],[X,Y,2],[X,Y,3]]) :-
	tripletPos(X),
	tripletPos(Y),
	tripletPos(Z).

% defines the straight triplets
straightTriplet([[1,Y,Z],[2,Y,Z],[3,Y,Z]]) :-
	tripletPos(Y),
	tripletPos(Z).
straightTriplet([[X,1,Z],[X,2,Z],[X,3,Z]]) :-
	tripletPos(X),
	tripletPos(Z).
straightTriplet([[X,Y,1],[X,Y,2],[X,Y,3]]) :-
	tripletPos(X),
	tripletPos(Y).


  % defines the diagonal triplets
% from corners
diagTriplet([[1,1,1],[2,2,2],[3,3,3]]).
diagTriplet([[1,3,1],[2,2,2],[3,1,3]]).
diagTriplet([[1,3,3],[2,2,2],[3,1,1]]).
diagTriplet([[1,1,3],[2,2,2],[3,3,1]]).
% from edges
diagTriplet([[X,1,3],[X,2,2],[X,3,1]]) :-
	tripletPos(X).
diagTriplet([[X,1,1],[X,2,2],[X,3,3]]) :-
	tripletPos(X).
diagTriplet([[1,Y,3],[2,Y,2],[3,Y,1]]) :-
	tripletPos(Y).
diagTriplet([[1,Y,1],[2,Y,2],[3,Y,3]]) :-
	tripletPos(Y).
diagTriplet([[1,3,Z],[2,2,Z],[3,1,Z]]) :-
	tripletPos(Z).
diagTriplet([[1,1,Z],[2,2,Z],[3,3,Z]]) :-
	tripletPos(Z).


% returns the player's chosen square for the move
playerMove(Board, [X, Y, Z]) :-
	write('Choose a layer (1, 2 or 3) '),
	getChar(InZ),
	validateLayer(InZ, SquZ),
	write('Choose a column (1, 2 or 3) '),
	getChar(InX),
	validateColumn(InX, SquX),
	write('Choose a row (1, 2 or 3) '),
	getChar(InY),
	validateRow(InY, SquY),
	validateSquare(Board, [SquX, SquY, SquZ], [X, Y, Z]),
	nl.

testPlayerMove :-
	Board = [[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,x,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]],
	drawBoard(Board),
	playerMove(Board, Square),
	write('Square chosen: '), write(Square).


% takes input of a single character
getChar(Char) :-
	get_code(ASCII),
	peek_code(Newline),
	validateChar(ASCII, Newline, Char).


% checks a single character has been given
% 1 - a single character has been given
validateChar(ASCII, Newline, Char) :-
	newline(Newline),
	name(Char, [ASCII]),
	toEndOfInput.
% 2 - a single character has not been given, retry input
validateChar(_ASCII, Newline, Char) :-
	\+ newline(Newline),
	write('Single character not found, retry input: '),
	toEndOfInput,
	getChar(Char).


% skips to the end of the player's input (when they pressed enter)
toEndOfInput :-
	newline(Newline),
	skip(Newline).


% checks the given square is a possible move for the player
% 1 - the given square is empty
validateSquare(Board, [X, Y, Z], [X, Y, Z]) :-
	marked(Board, [X, Y, Z], nil).
% 2 - the given square is not empty, get another
validateSquare(Board, [X, Y, Z], [NewX, NewY, NewZ]) :-
	\+ marked(Board, [X, Y, Z], nil),
	write('Square already taken, enter another.'), nl,
	playerMove(Board, [NewX, NewY, NewZ]).


% ensures the layer chosen is a possible layer
% 1 - the value given is 1, 2 or 3
validateLayer(Z, Z) :-
	tripletPos(Z).
% 2 - the value given is not valid, get another
validateLayer(Z, NewZ) :-
	\+ tripletPos(Z),
	write('Layer not chosen, choose a layer (1, 2 or 3) '),
	getChar(InZ),
	validateLayer(InZ, NewZ).


% ensures the row chosen is a possible row
% 1 - the value given is 1, 2 or 3
validateRow(Y, Y) :-
	tripletPos(Y).
% 2 - the value given is not valid, get another
validateRow(Y, NewY) :-
	\+ tripletPos(Y),
	write('Row not chosen, choose a row (1, 2 or 3) '),
	getChar(InY),
	validateRow(InY, NewY).


% ensures the column chosen is a possible column
% 1 - the value given is 1, 2 or 3
validateColumn(X, X) :-
	tripletPos(X).
% 2 - the value given is not valid, get another
validateColumn(X, NewX) :-
	\+ tripletPos(X),
	write('Column not chosen, choose a column (1, 2 or 3) '),
	getChar(InX),
	validateColumn(InX, NewX).


% returns player if the player wants to go first, comp otherwise
pickGo(First) :-
	write('Do you want the first go (y/n)? '),
	getChar(Char),
	valGo(Char, First).


% returns player if y is given, comp otherwise
valGo(y, player).
valGo(Char, comp) :-
	Char \== y.


% play the game
play :-
	pickGo(First),
	assertMarks(First),
	play(First, WhoWon),
	writeWinner(WhoWon),
	retractMarks(First).


% asserts the mark used for each player based on who's going first
% 1 - computer is going first
assertMarks(comp) :-
	assert(compMark(x)),
	assert(playerMark(0)).
% 2 - player is going first
assertMarks(player) :-
	assert(compMark(0)),
	assert(playerMark(x)).


% retracts the mark used for each player based on who went first
% 1 - computer went first
retractMarks(comp) :-
	retract(compMark(x)),
	retract(playerMark(0)).
% 2 - player went first
retractMarks(player) :-
	retract(compMark(0)),
	retract(playerMark(x)).


% write a message depending on who won the game
% 1 - the computer won
writeWinner(Mark) :-
	compMark(Mark),
	write('The computer won the game.').
% 1 - the player won
writeWinner(Mark) :-
	playerMark(Mark),
	write('The player won the game.').
% 1 - no-one won
writeWinner(nil) :- write('The game was a draw.').


% run the game, having chosen who goes first, and return the end board state and who won
% 1 - computer goes first
play(comp, WhoWon) :-
	startBoard(Board),
	compMove(Board, Square),
	write('Computer\'s go:'), nl, nl,
	mark(Board, Square, x, NewBoard),
	continue(player, NewBoard, WhoWon).
% 2 - player goes first
play(player, WhoWon) :-
	startBoard(Board),
	drawBoard(Board),
	playerMove(Board, Square),
	write('Your go:'), nl, nl,
	mark(Board, Square, x, NewBoard),
	continue(comp, NewBoard, WhoWon).


% initialise the playing board
startBoard(Board) :-
	Board = [[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		 [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]],
	nl,
	write('NEW GAME'), nl,
	write('========'), nl, nl.


% make a move, and recursively call until someone has won or the board is full.
% 1 - the game is finished
continue(_, Board, WhoWon) :-
	gameFinished(Board, WhoWon).
% 2 - it's the computer's turn
continue(comp, Board, WhoWon) :-
	compMove(Board, CompSquare),
	compMark(CompMark),
	write('Computer\'s go:'), nl, nl,
	mark(Board, CompSquare, CompMark, NewBoard),
	continue(player, NewBoard, WhoWon).
% 2 - it's the player's turn
continue(player, Board, WhoWon) :-
	playerMove(Board, PlayerSquare),
	playerMark(PlayerMark),
	write('Your go:'), nl, nl,
	mark(Board, PlayerSquare, PlayerMark, NewBoard),
	continue(comp, NewBoard, WhoWon).


% check if the game has finished
gameFinished(Board, WhoWon) :-
	getTriplets(Triplets),
	checkTriplets(Board, Triplets, WhoWon).

testGameFinished1 :-
	gameFinished([[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]], WhoWon),
	write(WhoWon).

testGameFinished2 :-
	gameFinished([[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
		      [[nil,nil,nil],[x,x,x],[nil,nil,nil]],
		      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]], WhoWon),
	write(WhoWon).


% checks the given list of triplets and returns nil if they're all full
% but no-one has won, or the mark of the winner if someone has won
% 1 - terminating - no triplet found with 3 of the same mark, but the board is full
checkTriplets(Board, [], nil) :-
	flatten(Board, FlatBoard),
	\+ member(nil, FlatBoard).
% 2 - terminating - found a triplet with 3 of the same mark
checkTriplets(Board, [Triplet|_Tail], WhoWon) :-
	numMarks(Board, Triplet, WhoWon, 3),
	WhoWon \== nil.
% 3 - recursive - found a triplet without 3 of the same mark
checkTriplets(Board, [Triplet|Tail], WhoWon) :-
	\+ numMarks(Board, Triplet, _, 3),
	checkTriplets(Board, Tail, WhoWon).
% 4 - recursive - found a triplet with 3 nils
checkTriplets(Board, [Triplet|Tail], WhoWon) :-
	numMarks(Board, Triplet, nil, 3),
	checkTriplets(Board, Tail, WhoWon).


% returns Board but with the given square marked with the given mark
% 1 - X is 1, look at the correct list to mark the square
mark([XList1, XList2, XList3], [1, SquareY, SquareZ], Mark, NewBoard) :-
	markXList(XList1, [SquareY, SquareZ], Mark, MarkedXList),
	NewBoard = [MarkedXList, XList2, XList3],
	drawBoard(NewBoard).
% 2 - X is 2, look at the correct list to mark the square
mark([XList1, XList2, XList3], [2, SquareY, SquareZ], Mark, [XList1, MarkedXList, XList3]) :-
	markXList(XList2, [SquareY, SquareZ], Mark, MarkedXList),
	NewBoard = [XList1, MarkedXList, XList3],
	drawBoard(NewBoard).
% 3 - X is 3, look at the correct list to mark the square
mark([XList1, XList2, XList3], [3, SquareY, SquareZ], Mark, [XList1, XList2, MarkedXList]) :-
	markXList(XList3, [SquareY, SquareZ], Mark, MarkedXList),
	NewBoard = [XList1, XList2, MarkedXList],
	drawBoard(NewBoard).

testMark1 :-
	mark([[[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
	      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]],
	      [[nil,nil,nil],[nil,nil,nil],[nil,nil,nil]]], [2,2,2], x, NewBoard),
	writeList(NewBoard).


% returns XList but with the given square marked with the given mark
% 1 - Y is 1, look at the correct list to mark the square
markXList([YList1, YList2, YList3], [1, SquareZ], Mark, [MarkedYList, YList2, YList3]) :-
	markYList(YList1, SquareZ, Mark, MarkedYList).
% 2 - Y is 2, look at the correct list to mark the square
markXList([YList1, YList2, YList3], [2, SquareZ], Mark, [YList1, MarkedYList, YList3]) :-
	markYList(YList2, SquareZ, Mark, MarkedYList).
% 3 - Y is 3, look at the correct list to mark the square
markXList([YList1, YList2, YList3], [3, SquareZ], Mark, [YList1, YList2, MarkedYList]) :-
	markYList(YList3, SquareZ, Mark, MarkedYList).


% returns YList but with the given square marked with the given mark
% 1 - Z is 1, mark the correct square
markYList([_Square1, Square2, Square3], 1, Mark, [Mark, Square2, Square3]).
% 2 - Z is 2, mark the correct square
markYList([Square1, _Square2, Square3], 2, Mark, [Square1, Mark, Square3]).
% 3 - Z is 3, mark the correct square
markYList([Square1, Square2, _Square3], 3, Mark, [Square1, Square2, Mark]).


% draw the board
drawBoard(Board) :-
	drawRow(Board, 1),
	drawRowPartition,
	drawRow(Board, 2),
	drawRowPartition,
	drawRow(Board, 3),
	write('x=1 x=2 x=3'),
	nl,
	write('    z=1             z=2             z=3'),
	nl,
	write('x is columns, y is rows, z is layers'),
	nl, nl, nl.


% draw a partition between rows
drawRowPartition :-
	write('-----------     -----------     -----------'),
	nl.


% draw a row of squares
drawRow(Board, Row) :-
	drawSquare(Board, [1,Row,1]),
	write('|'),
	drawSquare(Board, [2,Row,1]),
	write('|'),
	drawSquare(Board, [3,Row,1]),
	write('y='),
	write(Row),
	write('  '),
	drawSquare(Board, [1,Row,2]),
	write('|'),
	drawSquare(Board, [2,Row,2]),
	write('|'),
	drawSquare(Board, [3,Row,2]),
	write('     '),
	drawSquare(Board, [1,Row,3]),
	write('|'),
	drawSquare(Board, [2,Row,3]),
	write('|'),
	drawSquare(Board, [3,Row,3]),
	nl.


% draw a square
% 1 - the given square is not empty
drawSquare(Board, Square) :-
	marked(Board, Square, Mark),
	Mark \== nil,
	write(' '),
	write(Mark),
	write(' ').
% 2 - the given square is empty
drawSquare(Board, Square) :-
	marked(Board, Square, nil),
	write('   ').
